package utils;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;

import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil;

public class XMIUtils {

	/**
	 * Converts the given {@link EObject} model into its XMI representation.
	 * 
	 * @param model
	 *            The model to convert
	 * @return The XMI string representation of the given model
	 */
	public static String serializeEObjectToXMI(final EObject model, ResourceSet rs) {
		Resource resource = rs.createResource(org.eclipse.emf.common.util.URI.createFileURI("temp/model.xmi"));
		EcoreUtil.resolveAll(resource);
		if (model != null) {
			resource.getContents().add(model);
		}
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		try {
			resource.save(baos, null);
		} catch (IOException ioe) {
			ioe.printStackTrace();
		}
		String modelXMI = baos.toString();
		try {
			baos.close();
		} catch (IOException ioe) {
		}
		if (model != null) {
			resource.getContents().remove(model);
		}
		try {
			resource.delete(null);
		} catch (IOException e) {
			e.printStackTrace();
		}
		return modelXMI;
	}

	/**
	 * Converts the given String (containing XMI data) to its {@link EObject} representation.
	 * 
	 * @param xmiContents
	 *            The string containing the XMI to convert
	 * @return The EObject representation of the XMI data
	 */
	public static EObject deserializeXMIString(final String xmiContents) {
		File tempFile = createTempFile("ModelData", ".xmi", xmiContents);
		if (tempFile == null) {
			return null;
		}
		EObject model = deserializeFromXMIFile(tempFile, true);
		return model;
	}

	/**
	 * Reads the contents of the given XMI file and constructs the contained EObject model.
	 * 
	 * @param xmiFile
	 *            The file to deserialize
	 * @return The EObject representation of the XMI contents.
	 */
	public static EObject deserializeFromXMIFile(final File xmiFile) {
		return deserializeFromXMIFile(xmiFile, false);
	}

	/**
	 * Reads the contents of the given XMI file and constructs the contained EObject model.
	 * 
	 * @param xmiFile
	 *            The file to deserialize
	 * @param deleteResource
	 *            If this is set to <code>true</code>, the resulting EObject will be removed from XMIResource and the
	 *            resource (including the file itself) will be deleted after deserializing the EObject model from it.
	 * @return The EObject representation of the XMI contents.
	 */
	public static EObject deserializeFromXMIFile(final File xmiFile, final boolean deleteResource) {
		ResourceSet resourceSet = new ResourceSetImpl();
		Resource resource = resourceSet.createResource(URI.createFileURI(xmiFile.getAbsolutePath()));
		try {
			resource.load(null);
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		}
		EObject model = null;
		if (resource.getContents() != null && resource.getContents().size() > 0) {
			EcoreUtil.resolveAll(resource);
			model = resource.getContents().get(0);
		}
		if (deleteResource) {
			resource.getContents().remove(model);
			try {
				resource.delete(null);
			} catch (IOException e) {
				e.printStackTrace();
			}
		}
		return model;
	}

	/**
	 * Creates a temporary file with the given name prefix, the given name suffix and the given file contents.
	 * 
	 * @param namePrefix
	 *            The prefix of the name of the temporary file
	 * @param nameSuffix
	 *            The suffix of the name of the temporary file
	 * @param contents
	 *            The contents of the temporary file
	 * @return The temporary file, as specified, or <code>null</code> if an error occured
	 */
	public static File createTempFile(final String namePrefix, final String nameSuffix, final String contents) {
		File tempFile = null;
		FileWriter writer = null;
		BufferedWriter bWriter = null;
		boolean writtenSuccessfully = false;
		try {
			tempFile = File.createTempFile(namePrefix, nameSuffix);
			writer = new FileWriter(tempFile);
			bWriter = new BufferedWriter(writer);
			bWriter.write(contents);
			bWriter.flush();
			writtenSuccessfully = true;
		} catch (IOException e) {
			e.printStackTrace();
			return null;
		} finally {
			if (bWriter != null) {
				try {
					bWriter.close();
				} catch (IOException ioe) {
				}
			}
			if (writer != null) {
				try {
					writer.close();
				} catch (IOException ioe) {
				}
			}
		}
		if (!writtenSuccessfully) {
			return null;
		} else {
			return tempFile;
		}
	}

}